#include "MainWindow.h"
#include "ui_AdaptiveCutaways.h"
#include <RenderWidget.h>
#include <QFileDialog.h>
#include <QListWidgetItem>
#include <QListWidget>

MainWindow::MainWindow(QWidget *parent)
	: QMainWindow(parent), ui(new Ui::MainWindow)
{
	ui->setupUi(this);

	GLWidget = new RenderWidget(this);

	loadScene();

	// Model Paths
	//modelsOfScene.push_back(QString("models/test-sofa.obj"));
	//modelsOfScene.push_back(QString("models/sphere.obj"));
	//models.push_back(QString("models/velociraptor_mesh_materials.dae"));
	//models.push_back(QString("models/furniture.obj"));



	ui->splitter->setStretchFactor(1, 2);
	ui->gridLayout_rendering->addWidget(GLWidget);

	connect(ui->modelListWidget, SIGNAL(itemChanged(QListWidgetItem*)),this, SLOT(setCutable(QListWidgetItem*)));
	connect(ui->pushButton, SIGNAL(clicked()), this, SLOT(loadScene()));
	connect(ui->edgeComp, SIGNAL(stateChanged(int)), this, SLOT(setEdgeComp()));
	connect(ui->dirConst, SIGNAL(stateChanged(int)), this, SLOT(setDirConst()));
	connect(ui->angleSlider, SIGNAL(valueChanged(int)), this, SLOT(setAngle(int)));
	connect(ui->alphaSlider, SIGNAL(valueChanged(int)), this, SLOT(setAlpha(int)));
}

void MainWindow::setCutable(QListWidgetItem* model) {
	qDebug() << "MainWindow: setCutable()";
	if (model->checkState() == Qt::Checked) { // set this model as object of interest 
		GLWidget->setCutable("models/" + directory.dirName() + "/" + model->text(), false);
	}
	else { //return to normal object (no object of interest anymore)
		GLWidget->setCutable("models/" + directory.dirName() + "/" + model->text(), true);
	}

	GLWidget->update();
}

void MainWindow::setAngle(int angle)
{
	qDebug() << "Change angle to " << angle;
	GLWidget->setCutAngle(angle);
	GLWidget->update();
}

void MainWindow::setAlpha(int alpha)
{
	qDebug() << "Change alpha to " << alpha/100.0f;
	GLWidget->setCutAreaAlpha(alpha/100.0f);
	GLWidget->update();
}

void MainWindow::setEdgeComp()
{

	// Directional constraint only possible with edge compression
	if (ui->edgeComp->checkState() == Qt::Checked)
	{
		qDebug() << "Set edge compression: " << true;
		GLWidget->setEdgeCompression(true);
		ui->dirConst->setEnabled(true);
	}
	else
	{
		qDebug() << "Set edge compression: " << false;
		GLWidget->setEdgeCompression(false);
		ui->dirConst->setEnabled(false);
	}

	GLWidget->update();
}

void MainWindow::setDirConst()
{
	if (ui->dirConst->isEnabled())
	{
		if (ui->dirConst->checkState() == Qt::Checked)
		{
			qDebug() << "Set directional constraint: " << true;
			GLWidget->setDirectionalConstraint(true);
		}
		else
		{
			qDebug() << "Set directional constraint: " << false;
			GLWidget->setDirectionalConstraint(false);
		}

		GLWidget->update();
	}
}

// slot for "Load Scene" button
void MainWindow::loadScene() {

	modelsOfScene.clear();
	ui->modelListWidget->clear();

	dir = QFileDialog::getExistingDirectory(this, tr("Select Folder"), "./models", QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
	//qDebug() << dir;
	directory = QDir(dir);
	ui->lineEdit->setText(directory.absolutePath());
	//qDebug() << directory;
	
	modelList = directory.entryList(QStringList() << "*.obj" << "*.OBJ", QDir::Files);
	
	QListWidgetItem *item;
	
	foreach(QString model, modelList) {
		modelsOfScene.push_back("models/" + directory.dirName() + "/" + model);
		qDebug() << "models/" + directory.dirName() + "/" + model;

		item = new QListWidgetItem(model);
		item->setFlags(item->flags() | Qt::ItemIsUserCheckable);
		item->setCheckState(Qt::Unchecked);
		ui->modelListWidget->addItem(item);
	}

	GLWidget->setFilepaths(modelsOfScene);
	if (GLWidget->isInitialized()) {
		GLWidget->update();
	}
}

void MainWindow::closeEvent(QCloseEvent *event) {
	QMainWindow::closeEvent(event);
}

MainWindow::~MainWindow()
{
	delete GLWidget;
	delete ui;
}
