#pragma once

#include <qmainwindow.h>
#include <QListWidgetItem>
#include <QDir>


class RenderWidget;

namespace Ui {
	class MainWindow;
}

/**
* MainWindow represents the main user interface window. 
* It contains and manages all GUI components created with Qt. 
*/
class MainWindow : public QMainWindow
{
	Q_OBJECT

public:
	
	explicit MainWindow(QWidget *parent = 0);
	~MainWindow();

protected:
	/**
	* Implementation of the parent method. Called when the window is going to be closed.
	*/
	void closeEvent(QCloseEvent *event);

private:
	Ui::MainWindow *ui;
	RenderWidget *GLWidget;

	QString sceneFolder;
	QString dir;
	QDir directory;
	QStringList modelList;
	std::vector<QString> modelsOfScene;


private slots:

	/**
	* Loads the scene with all models of selected directory.
	*/
	void loadScene();
	
	/**
	* Changes the cut-able state of a model according its corresponding check box state.
	* @param model The object which cut-able state has changed.
	*/
	void setCutable(QListWidgetItem* model);
	
	/**
	* Sets the cutaway angle according the horizontal slider when it has changed.
	*/
	void setAngle(int angle);
	
	/**
	* Sets the alpha value of the cut areas according the horizontal slider when it has changed.
	*/
	void setAlpha(int alpha);

	/**
	* Enables or disables edge compression according to the corresponding check box.
	*/
	void setEdgeComp();

	/**
	* Enables or disables directional constraints according to the corresponding check box.
	*/
	void setDirConst();
};
