#include "CutawayBuffer.h"

#include <QDebug>

CutawayBuffer::CutawayBuffer(int screenWidth, int sreenHeight) :
	m_width(screenWidth), m_height(sreenHeight) 
{

	this->initializeOpenGLFunctions();

	init();

}

CutawayBuffer::~CutawayBuffer() 
{
	m_vbo.destroy();
	m_vao.destroy();
	glDeleteFramebuffers(1, &m_fbo);
	glDeleteTextures(1, &m_colorTexture);
	glDeleteTextures(1, &m_depthTexture);
}

void CutawayBuffer::init()
{

	// Create framebuffer object
	glGenFramebuffers(1, &m_fbo);

	// Create color texture
	glGenTextures(1, &m_colorTexture);
	glBindTexture(GL_TEXTURE_2D, m_colorTexture);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA16F, m_width, m_height, 0, GL_RGBA, GL_FLOAT, NULL);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
	glBindTexture(GL_TEXTURE_2D, 0);

	// Create depth texture
	glGenTextures(1, &m_depthTexture);
	glBindTexture(GL_TEXTURE_2D, m_depthTexture);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, m_width, m_height, 0, GL_DEPTH_COMPONENT, GL_FLOAT, 0);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
	GLfloat borderColor[] = { 1.0, 1.0, 1.0, 1.0 };
	glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, borderColor);

	// Add texture attachements to framebuffer
	glBindFramebuffer(GL_FRAMEBUFFER, m_fbo);
	glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, m_colorTexture, 0);

	glFramebufferTexture2D(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_TEXTURE_2D, m_depthTexture, 0);
	//glDrawBuffer(GL_NONE);
	//glReadBuffer(GL_NONE);

	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE) {

		std::cout << "ERROR: Framebuffer is not complete! " << glCheckFramebufferStatus(GL_FRAMEBUFFER) << std::endl;
	}
	glBindFramebuffer(GL_FRAMEBUFFER, 0);

	
}

void CutawayBuffer::bind()
{
	glBindFramebuffer(GL_FRAMEBUFFER, m_fbo);
	//glClearColor(.9f, .9f, .93f, 1.0f);
	//glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}


void CutawayBuffer::bindTexturesToShader(QOpenGLShaderProgram* shader, int textureUnit)
{

	shader->bind();

	// Activate color texture
	glActiveTexture(GL_TEXTURE0 + textureUnit);
	glBindTexture(GL_TEXTURE_2D, m_colorTexture);
	std::string name = ("texture" + std::to_string(textureUnit));
	shader->setUniformValue(name.c_str(), textureUnit);

	// Activate depth texture
	glActiveTexture(GL_TEXTURE0 + (textureUnit+1));
	glBindTexture(GL_TEXTURE_2D, m_depthTexture);
	name = ("depthMap" + std::to_string(textureUnit));
	shader->setUniformValue(name.c_str(), textureUnit+1);

}

void CutawayBuffer::bindColorTexToShader(QOpenGLShaderProgram * shader)
{
	shader->bind();

	// Activate color texture
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, m_colorTexture);
	shader->setUniformValue("texture0", 0);
}


void CutawayBuffer::unbind()
{
	glBindFramebuffer(GL_FRAMEBUFFER, 0);
	//glDrawBuffer(GL_BACK);

}

void CutawayBuffer::draw() 
{

	if (!m_vao.isCreated())
	{
		GLfloat quadVertices[] = {
			// Positions        // Texture Coords
			-1.0f, 1.0f, 0.0f, 0.0f, 1.0f,
			-1.0f, -1.0f, 0.0f, 0.0f, 0.0f,
			1.0f, 1.0f, 0.0f, 1.0f, 1.0f,
			1.0f, -1.0f, 0.0f, 1.0f, 0.0f,
		};
			
		// Setup vertex array object
		m_vao.create();
		m_vao.bind();

		// Create buffer and copy the vertex data to it
		m_vbo.create();
		m_vbo.setUsagePattern(QOpenGLBuffer::StaticDraw);
		m_vbo.bind();
		m_vbo.allocate(&quadVertices, sizeof(quadVertices));

		glEnableVertexAttribArray(0);
		glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 5 * sizeof(GLfloat), (GLvoid*)0);
		glEnableVertexAttribArray(1);
		glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 5 * sizeof(GLfloat), (GLvoid*)(3 * sizeof(GLfloat)));
	}

	// Draw quad to screen
	m_vao.bind();
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	m_vao.release();
	
}


void CutawayBuffer::resize(int width, int height)
{
	m_width = width;
	m_height = height;

	init();
}