#version 430 core

#define M_PI 3.14159265358979323846

uniform bool final;
uniform bool edgeCompression;
uniform bool directionalConstraint;

uniform float theta;
uniform float zNear;
uniform float zFar;

uniform sampler2D texture0; // screen positins of max depth
uniform sampler2D depthMap0;

uniform int k; // jump length
uniform vec2 imgSize; // width, height

out vec4 outColor;

void main(void)
{

	// Current pixel
	vec2 p = gl_FragCoord.xy / imgSize; // [0 imgSize] -> [0 1] 
	float zp = texture(depthMap0, p).r;
	
	// Steps of length k from p in all directions
	vec2 steps[9] = vec2[9](vec2(-k, k), 	vec2(0, k), 	vec2(k, k), 
							vec2(-k, 0), 	vec2(0, 0), 	vec2(k, 0), 
							vec2(-k, -k), 	vec2(0, -k), 	vec2(k, -k) );
	
	
	// Scale for slope m with Perspective Compensation	
	float scale = (zNear + zFar) / (zNear - zFar);
	
	// Maximize cutaway function C(p,q) = z(p) - m|q-p|
	float maxC = -1.0;
	float newDepth = zp;
	vec4 newTextureSample = texture(texture0, p);
	
	
	
	for (int i = 0; i < 9; i++)
	{
		vec2 coords = (gl_FragCoord.xy + steps[i]) / imgSize; 
		bvec2 greater0 = greaterThanEqual(coords,vec2(0.0));
		bvec2 less1 = lessThanEqual(coords,vec2(1.0));
		
		// Compare only if coordinates are valid
		if (greater0.x && greater0.y && less1.x && less1.y)
		{
			float zq = texture(depthMap0, coords).r;
			vec4 textureSample = texture(texture0, coords);
			vec2 q = textureSample.xy / imgSize;
			
			// Denormalize angles [0 1] -> [0 theta]
			float thetaEdge = textureSample.z * theta;
			float thetaDir = textureSample.w * theta;
			
			// Slopes for edge compression and directional constraints
			float mEdge = -(scale + zq) / tan(thetaEdge);
			float mDir = -(scale + zq) / tan(thetaDir);
								
			float alpha = clamp( dot(normalize(q-p), vec2(0.0f,1.0f)), 0.0f, 1.0f);
			float C = zq - ( (1.0f-alpha) * mEdge + alpha * mDir ) * distance(q,p);
			
			if (C > maxC)
			{
				maxC = C;
				newDepth = zq; // new max depth is at position q
				newTextureSample = textureSample; // save screen coords and angles of max depth
			
			}
		}
		
	}
	
	if (final)
	{
		//vec4 textureSample = texture(texture0, p);
		vec2 q = newTextureSample.xy / imgSize;
		float z = newDepth;
		
		// Denormalize angles [0 1] -> [0 theta]
		float thetaEdge = newTextureSample.z * theta;
		float thetaDir = newTextureSample.w * theta;
		
		// Slopes for edge compression and directional constraints
		float mEdge = -(scale + z) / tan(thetaEdge);
		float mDir = -(scale + z) / tan(thetaDir);
				
		float alpha = clamp( dot(normalize(q-p), vec2(0.0f,1.0f)), 0.0f, 1.0f);		
		newDepth = z - ( (1.0f-alpha) * mEdge + alpha * mDir ) * distance(q,p);
		
	}
	
	

	outColor = newTextureSample;
	gl_FragDepth = newDepth;
	
}
